@@ -76,13 +76,11 @@ imlib_render_str(ImlibImage * im, ImlibFont * fn, int drx, int dry,
    int                 w, h, ascent;
    ImlibImage         *im2;
    DATA32             *data, col;
-   int                 nx, ny, tmp;
-   int                 inset;
-   
-   inset = imlib_font_query_inset(fn, text);
-   imlib_font_query_size(fn, text, &w, &h);
-   w += inset;
-   
+   int                 nx, ny;
+
+   imlib_font_query_advance(fn, text, &w, NULL);
+   h = imlib_font_max_ascent_get(fn) - imlib_font_max_descent_get(fn);
+
    data = malloc(w * h * sizeof(DATA32));
    if (!data)
       return;
@@ -101,7 +99,7 @@ imlib_render_str(ImlibImage * im, ImlibFont * fn, int drx, int dry,
 
    ascent = imlib_font_max_ascent_get(fn);
 
-   imlib_font_draw(im2, col, fn, 0, ascent, text, &nx, &ny, clx, cly, clw, clh);
+   imlib_font_draw(im2, col, fn, 0, ascent, text, &nx, &ny, 0, 0, w, h);
 
    /* OK, now we have small ImlibImage with text rendered, 
     * have to blend it on im */
