@@ -53,8 +53,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
    else
       iny = oy;
    /* go thru the XImage and convert */
-   if (xim->bits_per_pixel == 32)
-      depth = 32;
+   if ((depth == 24) && (xim->bits_per_pixel == 32))
+      depth = 25;	/* fake depth meaning 24 bit in 32 bpp ximage */
    /* data needs swapping */
 #define SWAP32(x) (x) = \
    ((((int)(x) & 0x000000ff ) << 24) |\
@@ -96,6 +96,7 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
 		      }
 		 }
 	     case 24:
+	     case 25:
 	     case 32:
 	       for (y = 0; y < h; y++)
 		 {
@@ -374,7 +375,7 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
                  }
             }
           break;
-       case 32:
+       case 25:
           if (bgr)
             {
                if (mxim)
@@ -449,6 +450,74 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
                  }
             }
           break;
+       case 32:
+          if (bgr)
+            {
+               if (mxim)
+                 {
+                    for (y = 0; y < h; y++)
+                      {
+                         src =
+                             (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                         ptr = data + ((y + iny) * ow) + inx;
+                         for (x = 0; x < w; x++)
+                           {
+                              pixel = SWAP32(*src);
+                              if (!XGetPixel(mxim, x, y))
+                                 pixel &= 0x00ffffff;
+                              *ptr++ = pixel;
+                              src++;
+                           }
+                      }
+                 }
+               else
+                 {
+                    for (y = 0; y < h; y++)
+                      {
+                         src =
+                             (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                         ptr = data + ((y + iny) * ow) + inx;
+                         for (x = 0; x < w; x++)
+                           {
+                              *ptr++ = SWAP32(*src);
+                              src++;
+                           }
+                      }
+                 }
+            }
+          else
+            {
+               if (mxim)
+                 {
+                    for (y = 0; y < h; y++)
+                      {
+                         src =
+                             (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                         ptr = data + ((y + iny) * ow) + inx;
+                         for (x = 0; x < w; x++)
+                           {
+                              pixel = *src++;
+                              if (!XGetPixel(mxim, x, y))
+                                 pixel &= 0x00ffffff;
+                              *ptr++ = pixel;
+                           }
+                      }
+                 }
+               else
+                 {
+                    for (y = 0; y < h; y++)
+                      {
+                         src =
+                             (DATA32 *) (xim->data + (xim->bytes_per_line * y));
+                         ptr = data + ((y + iny) * ow) + inx;
+                         for (x = 0; x < w; x++)
+                           {
+                              *ptr++ = *src++;
+                           }
+                      }
+                 }
+            }
+          break;
        default:
           break;
      }
@@ -461,11 +530,12 @@ char
 __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
                            Display * d, Drawable p, Pixmap m, Visual * v,
                            Colormap cm, int depth, int x, int y,
-                           int w, int h, char domask, char grab)
+                           int w, int h, char *pdomask, char grab)
 {
    XErrorHandler       prev_erh = NULL;
    XWindowAttributes   xatt, ratt;
    char                is_pixmap = 0, created_mask = 0, is_shm = 0, is_mshm = 0;
+   char                domask;
    int                 i;
    int                 src_x, src_y, src_w, src_h, origw, origh;
    int                 width, height, clipx, clipy;
@@ -474,6 +544,7 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
    static signed char  x_does_shm = -1;
    XColor              cols[256];
 
+   domask = (pdomask) ? *pdomask : 0;
    /* FIXME: oh isnt used - i wonder if there's a bug looming... */
    oh = 0;
    origw = w;
@@ -658,6 +729,12 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
      }
    if (!is_shm)
       xim = XGetImage(d, p, x, y, w, h, 0xffffffff, ZPixmap);
+   if (!xim)
+     {
+        if (grab)
+           XUngrabServer(d);
+	return 0;
+     }
 
    if ((m) && (domask))
      {
@@ -785,6 +862,16 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
       XFreePixmap(d, m);
    if (mxim)
       XDestroyImage(mxim);
+
+   if (pdomask)
+     {
+        /* Set domask according to whether or not we have useful alpha data */
+        if (xatt.depth == 32)
+           *pdomask = 1;
+        else if (!m)
+           *pdomask = 0;
+     }
+
    return 1;
 }
 
