@@ -26,12 +26,8 @@ __imlib_RenderGetPixel(Display * d, Drawable w, Visual * v, Colormap cm,
                        int depth, DATA8 r, DATA8 g, DATA8 b)
 {
    Context            *ct;
-   int                 actual_depth;
 
    ct = __imlib_GetContext(d, v, cm, depth);
-   actual_depth = depth;
-   if (depth == 16)
-      actual_depth = __imlib_XActualDepth(d, v);
 
    if (ct->palette)
      {
@@ -139,7 +135,7 @@ __imlib_RenderGetPixel(Display * d, Drawable w, Visual * v, Colormap cm,
    return 0;
 }
 
-void
+__hidden void
                     __imlib_generic_render(DATA32 * src, int jump, int w, int h, int dx, int dy,
                                            XImage * xim, Visual * v,
                                            Context * ct);
@@ -246,21 +242,22 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
                     int sx, int sy, int sw, int sh,
                     int dx, int dy, int dw, int dh,
                     char antialias, char hiq, char blend, char dither_mask,
-                    ImlibColorModifier * cmod, ImlibOp op)
+                    int mat, ImlibColorModifier * cmod, ImlibOp op)
 {
    XImage             *xim = NULL, *mxim = NULL;
    Context            *ct;
    DATA32             *buf = NULL, *pointer = NULL, *back = NULL;
    int                 y, h, hh, jump;
    static Display     *disp = NULL;
+   static int          last_depth = 0;
    static GC           gc = 0;
    static GC           gcm = 0;
    XGCValues           gcv;
    ImlibScaleInfo     *scaleinfo = NULL;
    int                 psx, psy, psw, psh;
-   int                 actual_depth = 0;
    char                shm = 0;
-   ImlibRGBAFunction   rgbaer, masker = NULL;
+   ImlibRGBAFunction   rgbaer;
+   ImlibMaskFunction   masker = NULL;
    ImlibBlendFunction  blender = NULL;
    int                 do_mmx;
 
@@ -312,9 +309,6 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
    dw = abs(dw);
    dh = abs(dh);
    ct = __imlib_GetContext(d, v, cm, depth);
-   actual_depth = depth;
-   if (depth == 16)
-      actual_depth = __imlib_XActualDepth(d, v);
    __imlib_RGBASetupContext(ct);
    if ((blend) && (IMAGE_HAS_ALPHA(im)))
      {
@@ -335,9 +329,6 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
            free(back);
         return;
      }
-   /* do a double check in 24/32bpp */
-   if ((xim->bits_per_pixel == 32) && (depth == 24))
-      actual_depth = 32;
    if (m)
      {
         mxim = __imlib_ProduceXImage(d, v, 1, dw, dh, &shm);
@@ -460,7 +451,7 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
         if (m)
            masker(pointer, jump,
                   ((DATA8 *) mxim->data) + (y * (mxim->bytes_per_line)),
-                  mxim->bytes_per_line, dw, hh, dx, dy + y);
+                  mxim->bytes_per_line, dw, hh, dx, dy + y, mat);
         h -= LINESIZE;
      }
    /* free up our buffers and poit tables */
@@ -470,25 +461,28 @@ __imlib_RenderImage(Display * d, ImlibImage * im,
       __imlib_FreeScaleInfo(scaleinfo);
    if (back)
       free(back);
-   /* if we changed diplays since last time... free old gc's */
-   if (disp != d)
+   /* if we changed diplays or depth since last time... free old gc */
+   if ((gc) && ((last_depth != depth) || (disp != d)))
      {
-        if (gc)
-           XFreeGC(disp, gc);
-        if (gcm)
-           XFreeGC(disp, gcm);
+        XFreeGC(disp, gc);
         gc = 0;
-        gcm = 0;
      }
    /* if we didnt have a gc... create it */
    if (!gc)
      {
         disp = d;
+        last_depth = depth;
         gcv.graphics_exposures = False;
         gc = XCreateGC(d, w, GCGraphicsExposures, &gcv);
      }
    if (m)
      {
+        /* if we changed diplays since last time... free old gc */
+        if ((gcm) && (disp != d))
+        {
+           XFreeGC(disp, gcm);
+           gcm = 0;
+        }
         if (!gcm)
           {
              gcv.graphics_exposures = False;
@@ -524,7 +518,7 @@ __imlib_RenderImageSkewed(Display * d, ImlibImage * im, Drawable w, Drawable m,
                           int sx, int sy, int sw, int sh, int dx, int dy,
                           int hsx, int hsy, int vsx, int vsy,
                           char antialias, char hiq, char blend,
-                          char dither_mask, ImlibColorModifier * cmod,
+                          char dither_mask, int mat, ImlibColorModifier * cmod,
                           ImlibOp op)
 {
    Context            *ct;
@@ -591,7 +585,7 @@ __imlib_RenderImageSkewed(Display * d, ImlibImage * im, Drawable w, Drawable m,
                                    cmod, op, 0, 0, 0, 0);
 
    __imlib_RenderImage(d, back, w, m, v, cm, depth, 0, 0, dw, dh,
-                       dx1, dy1, dw, dh, 0, hiq, 0, dither_mask, 0, OP_COPY);
+                       dx1, dy1, dw, dh, 0, hiq, 0, dither_mask, mat, 0, OP_COPY);
    __imlib_FreeImage(back);
 }
 
