@@ -8,17 +8,19 @@
 #define DM_X (8)
 #define DM_Y (8)
 
-void    __imlib_RGBASetupContext(Context *ct);
-void    __imlib_RGBA_init(void *rd, void *gd, void *bd, int depth, 
+__hidden void    __imlib_RGBASetupContext(Context *ct);
+__hidden void    __imlib_RGBA_init(void *rd, void *gd, void *bd, int depth, 
 			  DATA8 palette_type);
 
 typedef void (*ImlibRGBAFunction)(DATA32*, int, DATA8*,
 				  int, int, int, int, int);
-ImlibRGBAFunction
+typedef void (*ImlibMaskFunction)(DATA32*, int, DATA8*,
+				  int, int, int, int, int, int);
+__hidden ImlibRGBAFunction
 __imlib_GetRGBAFunction(int depth, 
 			unsigned long rm, unsigned long gm, unsigned long bm, 
 			char hiq, DATA8 palette_type);
-ImlibRGBAFunction
+__hidden ImlibMaskFunction
 __imlib_GetMaskFunction(char hiq);
 
 #ifdef DO_MMX_ASM
