@@ -1,17 +1,4 @@
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
-#include "common.h"
-#include <string.h>
-
-#include "image.h"
-
-char                load(ImlibImage * im, ImlibProgressFunction progress,
-                         char progress_granularity, char immediate_load);
-char                save(ImlibImage * im, ImlibProgressFunction progress,
-                         char progress_granularity);
-void                formats(ImlibLoader * l);
+#include "loader_common.h"
 
 #define SWAP32(x) (x) = \
 ((((x) & 0x000000ff ) << 24) |\
@@ -47,6 +34,11 @@ load(ImlibImage * im, ImlibProgressFunction progress,
            fclose(f);
            return 0;
         }
+      if ((w < 1) || (h < 1) || (w > 8192) || (h > 8192))
+	{
+           fclose(f);
+           return 0;
+	}
       im->w = w;
       im->h = h;
       if (!im->format)
