@@ -11,19 +11,7 @@
  * Version: 2004-08-28
  *------------------------------------------------------------------------------*/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include "common.h"
-#include "image.h"
-
-char    load(ImlibImage *im, ImlibProgressFunction progress, char progress_granularity, char immediate_load);
-#if 0
-char    save(ImlibImage *im, ImlibProgressFunction progress, char progress_granularity);
-#endif
-void    formats(ImlibLoader *l);
-
+#include "loader_common.h"
 
 #define L2RLONG(a) ((((long)((a)[0]) & 0xff) << 24) + (((long)((a)[1]) & 0xff) << 16) + (((long)((a)[2]) & 0xff) << 8) + ((long)((a)[3]) & 0xff))
 #define L2RWORD(a) ((((long)((a)[0]) & 0xff) << 8) + ((long)((a)[1]) & 0xff))
@@ -53,13 +41,6 @@ typedef struct ILBM {
     int     rle;
 } ILBM;
 
-static void freeilbm(ILBM *);
-static int loadchunks(char *, ILBM *, int);
-static void bodyrow(unsigned char *, int, ILBM *);
-static unsigned char scalegun(unsigned char, int);
-static void scalecmap(ILBM *);
-static void deplane(DATA32 *, int, ILBM *, unsigned char **);
-
 
 /*------------------------------------------------------------------------------
  * Frees memory allocated as part of an ILBM structure.
@@ -268,7 +249,7 @@ int     i;
 static void deplane(DATA32 *row, int w, ILBM *ilbm, unsigned char *plane[])
 {
 unsigned long l;
-int     i, o, sl, x;
+int     i, o, x;
 unsigned char bit, r, g, b, a, v, h, *pal;
 
     pal = NULL;
@@ -395,7 +376,7 @@ unsigned char bit, r, g, b, a, v, h, *pal;
 char    load(ImlibImage *im, ImlibProgressFunction progress, char progress_granularity, char immediate_load)
 {
 char    *env;
-int     cancel, full, i, j, k, n, ok, y, z, gran, nexty, prevy;
+int     cancel, full, i, n, ok, y, z, gran, nexty, prevy;
 unsigned char *plane[40];
 ILBM    ilbm;
 
@@ -421,7 +402,10 @@ ILBM    ilbm;
 
         im->w = L2RWORD(ilbm.bmhd.data);
         im->h = L2RWORD(ilbm.bmhd.data + 2);
-        if (im->w <= 0 || im->h <= 0) ok = 0;
+	if ((im->w < 1) || (im->h < 1) || (im->w > 8192) || (im->h > 8192))
+	  {
+	     ok = 0;
+	  }
 
         ilbm.depth = ilbm.bmhd.data[8];
         if (ilbm.depth < 1 || (ilbm.depth > 8 && ilbm.depth != 24 && ilbm.depth != 32)) ok = 0; /* Only 1 to 8, 24, or 32 planes. */
