@@ -1,17 +1,9 @@
 /* To do: */
 /* o Need code to handle tiff with different orientations */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
-#include "common.h"
-#include <string.h>
-
+#include "loader_common.h"
 #include <setjmp.h>
 #include <stdarg.h>
-#include <stdio.h>
-#include "image.h"
 #include <tiffio.h>
 
 /* This is a wrapper data structure for TIFFRGBAImage, so that data can be */
@@ -31,21 +23,8 @@ struct TIFFRGBAImage_Extra {
 
 typedef struct TIFFRGBAImage_Extra TIFFRGBAImage_Extra;
 
-static void         put_contig_and_raster(TIFFRGBAImage *, uint32 *,
-                                          uint32, uint32, uint32, uint32, int32,
-                                          int32, unsigned char *);
-static void         put_separate_and_raster(TIFFRGBAImage *, uint32 *, uint32,
-                                            uint32, uint32, uint32, int32,
-                                            int32, unsigned char *,
-                                            unsigned char *, unsigned char *,
-                                            unsigned char *);
 static void         raster(TIFFRGBAImage_Extra * img, uint32 * raster, uint32 x,
                            uint32 y, uint32 w, uint32 h);
-char                load(ImlibImage * im, ImlibProgressFunction progress,
-                         char progress_granularity, char immediate_load);
-char                save(ImlibImage * im, ImlibProgressFunction progress,
-                         char progress_granularity);
-void                formats(ImlibLoader * l);
 
 static void
 put_contig_and_raster(TIFFRGBAImage * img, uint32 * rast,
@@ -75,11 +54,11 @@ static void
 raster(TIFFRGBAImage_Extra * img, uint32 * rast,
        uint32 x, uint32 y, uint32 w, uint32 h)
 {
-   uint32              image_width, image_height;
+   int                 image_width, image_height;
    uint32             *pixel, pixel_value;
    int                 i, j, dy, rast_offset;
    DATA32             *buffer_pixel, *buffer = img->image->data;
-   int                 alpha_premult = (EXTRASAMPLE_UNASSALPHA==img->rgba.alpha);
+   int                 alpha_premult;
 
    image_width = img->image->w;
    image_height = img->image->h;
@@ -91,6 +70,8 @@ raster(TIFFRGBAImage_Extra * img, uint32 * rast,
    /* I don't understand why, but that seems to be what's going on. */
    /* libtiff needs better docs! */
 
+   if (img->rgba.alpha == EXTRASAMPLE_UNASSALPHA)
+     alpha_premult = 1;
    for (i = y, rast_offset = 0; i > dy; i--, rast_offset--)
      {
         pixel = rast + (rast_offset * image_width);
@@ -204,6 +185,12 @@ load(ImlibImage * im, ImlibProgressFunction progress,
    rgba_image.image = im;
    im->w = width = rgba_image.rgba.width;
    im->h = height = rgba_image.rgba.height;
+   if ((width < 1) || (height < 1) || (width > 8192) || (height > 8192))
+     {
+	TIFFRGBAImageEnd((TIFFRGBAImage *) & rgba_image);
+        TIFFClose(tif);
+        return 0;
+     }
    rgba_image.num_pixels = num_pixels = width * height;
    if (rgba_image.rgba.alpha != EXTRASAMPLE_UNSPECIFIED)
       SET_FLAG(im->flags, F_HAS_ALPHA);
@@ -224,9 +211,9 @@ load(ImlibImage * im, ImlibProgressFunction progress,
           {
              fprintf(stderr, "imlib2-tiffloader: Out of memory\n");
 
-             if (!rast)
+             if (rast)
                 _TIFFfree(rast);
-             if (!im->data)
+             if (im->data)
                {
                   free(im->data);
                   im->data = NULL;
@@ -397,8 +384,9 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
 
    if (has_alpha)
      {
+	uint16 extras[] = { EXTRASAMPLE_ASSOCALPHA };
         TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 4);
-        TIFFSetField(tif, TIFFTAG_EXTRASAMPLES, EXTRASAMPLE_ASSOCALPHA);
+	TIFFSetField(tif, TIFFTAG_EXTRASAMPLES, 1, extras);
      }
    else
      {
