@@ -1,21 +1,7 @@
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
-#include "common.h"
-#include <stdio.h>
+#include "loader_common.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <string.h>
-
-#include "image.h"
-
-char                load(ImlibImage * im, ImlibProgressFunction progress,
-                         char progress_granularity, char immediate_load);
-char                save(ImlibImage * im, ImlibProgressFunction progress,
-                         char progress_granularity);
-void                formats(ImlibLoader * l);
 
 static FILE        *rgb_txt = NULL;
 
@@ -108,7 +94,7 @@ load(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity,
    FILE               *f;
 
    int                 pc, c, i, j, k, w, h, ncolors, cpp, comment, transp,
-       quote, context, len, done, r, g, b;
+       quote, context, len, done, r, g, b, backslash;
    char               *line, s[256], tok[128], col[256];
    int                 lsz = 256;
    struct _cmap {
@@ -163,6 +149,7 @@ load(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity,
    pixels = 0;
    count = 0;
    line = malloc(lsz);
+   backslash = 0;
    memset(lookup, 0, sizeof(lookup));
    while (!done)
      {
@@ -210,19 +197,19 @@ load(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity,
                             xpm_parse_done();
                             return 0;
                          }
-                       if ((w > 32767) || (w < 1))
+                       if ((w > 8192) || (w < 1))
                          {
                             fprintf(stderr,
-                                    "IMLIB ERROR: Image width > 32767 or < 1 pixels for file\n");
+                                    "IMLIB ERROR: Image width > 8192 or < 1 pixels for file\n");
                             free(line);
                             fclose(f);
                             xpm_parse_done();
                             return 0;
                          }
-                       if ((h > 32767) || (h < 1))
+                       if ((h > 8192) || (h < 1))
                          {
                             fprintf(stderr,
-                                    "IMLIB ERROR: Image height > 32767 or < 1 pixels for file\n");
+                                    "IMLIB ERROR: Image height > 8192 or < 1 pixels for file\n");
                             free(line);
                             fclose(f);
                             xpm_parse_done();
@@ -245,6 +232,32 @@ load(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity,
 
                        per_inc = 100.0 / (((float)w) * h);
 
+                       if (im->loader || immediate_load || progress)
+                         {
+                            im->data =
+                                (DATA32 *) malloc(sizeof(DATA32) * im->w *
+                                                  im->h);
+                            if (!im->data)
+                              {
+                                 free(cmap);
+                                 free(line);
+                                 fclose(f);
+                                 xpm_parse_done();
+                                 return 0;
+                              }
+                            ptr = im->data;
+                            end = ptr + (sizeof(DATA32) * w * h);
+                            pixels = w * h;
+                         }
+                       else
+                         {
+                            free(cmap);
+                            free(line);
+                            fclose(f);
+                            xpm_parse_done();
+                            return 1;
+                         }
+
                        j = 0;
                        context++;
                     }
@@ -362,32 +375,6 @@ load(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity,
                          {
                             UNSET_FLAG(im->flags, F_HAS_ALPHA);
                          }
-
-                       if (im->loader || immediate_load || progress)
-                         {
-                            im->data =
-                                (DATA32 *) malloc(sizeof(DATA32) * im->w *
-                                                  im->h);
-                            if (!im->data)
-                              {
-                                 free(cmap);
-                                 free(line);
-                                 fclose(f);
-                                 xpm_parse_done();
-                                 return 0;
-                              }
-                            ptr = im->data;
-                            end = ptr + (sizeof(DATA32) * w * h);
-                            pixels = w * h;
-                         }
-                       else
-                         {
-                            free(cmap);
-                            free(line);
-                            fclose(f);
-                            xpm_parse_done();
-                            return 1;
-                         }
                     }
                   else
                     {
@@ -645,7 +632,18 @@ load(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity,
                 c = 32;
              else if (c > 127)
                 c = 127;
-             line[i++] = c;
+	     if ( c=='\\' ) {
+	       if ( ++backslash<2 ) {
+		 line[i++] = c;
+	       }
+	       else {
+		 backslash = 0;
+	       }
+	     }
+	     else {
+	       backslash = 0;
+	       line[i++] = c;
+	     }
           }
         if (i >= lsz)
           {
